﻿// CircleShifter.cs by Charles Petzold, June 2009
using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;

namespace ChartingLib
{
    public class CircleShifter : Decorator
    {
        public static readonly DependencyProperty BasePointProperty =
            DependencyProperty.Register("BasePoint",
                typeof(Point),
                typeof(CircleShifter),
                new FrameworkPropertyMetadata(new Point(), FrameworkPropertyMetadataOptions.AffectsArrange));

        public static readonly DependencyProperty RotateTransformProperty =
            DependencyProperty.Register("RotateTransform",
                typeof(RotateTransform),
                typeof(CircleShifter),
                new FrameworkPropertyMetadata(new RotateTransform(), FrameworkPropertyMetadataOptions.AffectsArrange));

        public Point BasePoint
        {
            set { SetValue(BasePointProperty, value); }
            get { return (Point)GetValue(BasePointProperty); }
        }

        public RotateTransform RotateTransform
        {
            set { SetValue(RotateTransformProperty, value); }
            get { return (RotateTransform)GetValue(RotateTransformProperty); }
        }

        protected override Size ArrangeOverride(Size arrangeSize)
        {
            Point rotatedPoint = RotateTransform.Transform(BasePoint);
            Point arrangePoint = new Point();
            double angle = RotateTransform.Angle;

            angle %= 360;
            if (angle < 0)
                angle += 360;

            if (angle < 180)
                arrangePoint = new Point(rotatedPoint.X, 
                                         rotatedPoint.Y - (180 - angle) * Child.DesiredSize.Height / 180);
            else
                arrangePoint = new Point(rotatedPoint.X - Child.DesiredSize.Width,
                                         rotatedPoint.Y - (angle - 180) * Child.DesiredSize.Height / 180);

            Child.Arrange(new Rect(arrangePoint, Child.DesiredSize));
            return arrangeSize;
        }
    }
}
