﻿// Gizmo.cs by Charles Petzold, June 2009
using System;
using System.ComponentModel;

namespace ChartingLib
{
    public class Gizmo : INotifyPropertyChanged
    {
        string name;
        double revenues, scalingFactor, scaledRevenues;

        public event PropertyChangedEventHandler PropertyChanged;

        public string Name
        {
            set
            {
                if (value != name)
                {
                    name = value;
                    OnPropertyChanged("Name");
                }
            }
            get
            {
                return name;
            }
        }

        public double Revenues
        {
            set
            {
                if (value != revenues)
                {
                    revenues = value;
                    OnPropertyChanged("Revenues");
                    ScaledRevenues = ScalingFactor * revenues;
                }
            }
            get
            {
                return revenues;
            }
        }

        public double ScalingFactor
        {
            set
            {
                if (value != scalingFactor)
                {
                    scalingFactor = value;
                    OnPropertyChanged("ScalingFactor");
                    ScaledRevenues = scalingFactor * Revenues;
                }
            }
            get
            {
                return scalingFactor;
            }
        }

        public double ScaledRevenues
        {
            protected set
            {
                if (value != scaledRevenues)
                {
                    scaledRevenues = value;
                    OnPropertyChanged("ScaledRevenues");
                }
            }
            get
            {
                return scaledRevenues;
            }
        }

        protected virtual void OnPropertyChanged(string propertyName)
        {
            if (PropertyChanged != null)
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
        }
    }
}
