﻿// GizmoCollection.cs by Charles Petzold, June 2009
using System;
using System.Collections.ObjectModel;
using System.Collections.Specialized;
using System.ComponentModel;

namespace ChartingLib
{
    public class GizmoCollection : ObservableCollection<Gizmo>
    {
        Gizmo gizmoWithMaximumRevenues;
        double maximumRevenues;

        public event EventHandler MaximumRevenuesChanged;

        public double MaximumRevenues
        {
            set
            {
                if (value != maximumRevenues)
                {
                    maximumRevenues = value;

                    if (MaximumRevenuesChanged != null)
                        MaximumRevenuesChanged(this, EventArgs.Empty);
                }
            }
            get
            {
                return maximumRevenues;
            }
        }

        protected override void OnCollectionChanged(NotifyCollectionChangedEventArgs args)
        {
            base.OnCollectionChanged(args);

            if (args.NewItems != null)
                foreach (INotifyPropertyChanged item in args.NewItems)
                    item.PropertyChanged += OnItemPropertyChanged;

            if (args.OldItems != null)
                foreach (INotifyPropertyChanged item in args.OldItems)
                    item.PropertyChanged -= OnItemPropertyChanged;

            base.OnCollectionChanged(args);

            CalculateMaximumRevenues();
        }

        protected override void ClearItems()
        {
            foreach (INotifyPropertyChanged item in Items)
                item.PropertyChanged -= OnItemPropertyChanged;

            base.ClearItems();
            CalculateMaximumRevenues();
        }

        void OnItemPropertyChanged(object sender,
                                   PropertyChangedEventArgs args)
        {
            Gizmo gizmo = sender as Gizmo;

            if (args.PropertyName == "Revenues")
            {
                if (gizmo == gizmoWithMaximumRevenues)
                {
                    if (gizmo.Revenues > MaximumRevenues)
                        MaximumRevenues = gizmo.Revenues;
                    else
                        CalculateMaximumRevenues();
                }
                else if (gizmo.Revenues > MaximumRevenues)
                        CalculateMaximumRevenues();
            }

        }

        void CalculateMaximumRevenues()
        {
            gizmoWithMaximumRevenues = null;
            MaximumRevenues = 0;

            foreach (Gizmo gizmo in Items)
            {
                if (gizmo.Revenues >= MaximumRevenues)
                {
                    gizmoWithMaximumRevenues = gizmo;
                    MaximumRevenues = gizmo.Revenues;
                }
            }
        }
    }
}
