﻿// GizmoPresenter.cs by Charles Petzold, June 2009
using System;
using System.ComponentModel;
using System.Windows.Threading;

namespace ChartingLib
{
    public class GizmoPresenter : INotifyPropertyChanged
    {
        Random rand = new Random();
        double displayHeight, revenuesBase, halfRevenuesBase;
        GizmoCollection gizmos;

        public event PropertyChangedEventHandler PropertyChanged;

        public GizmoPresenter()
        {
            Gizmos = new GizmoCollection();
            Gizmos.MaximumRevenuesChanged += new EventHandler(OnMaximumRevenueChanged);

            for (int i = 0; i < 30; i++)
            {
                Gizmo gizmo = new Gizmo();
                gizmo.Name = ChartingLibHelper.GenerateName();
                gizmo.Revenues = 100000 * rand.NextDouble();
                Gizmos.Add(gizmo);
            }

            DispatcherTimer tmr = new DispatcherTimer();
            tmr.Tick += new EventHandler(OnTimerTick);
            tmr.Interval = TimeSpan.FromSeconds(1);
            tmr.Start();
        }

        public GizmoCollection Gizmos
        {
            set 
            {
                if (value != gizmos)
                {
                    gizmos = value;
                    OnPropertyChanged("Gizmos");
                }
            }
            get
            {
                return gizmos;
            }
        }

        public double DisplayHeight
        {
            set
            {
                if (value != displayHeight)
                {
                    displayHeight = value;
                    OnPropertyChanged("DisplayHeight");
                    SetScalingFactor();
                }
            }
            get
            {
                return displayHeight;
            }
        }

        public double RevenuesBase
        {
            protected set
            {
                if (value != revenuesBase)
                {
                    revenuesBase = value;
                    OnPropertyChanged("RevenuesBase");
                    HalfRevenuesBase = RevenuesBase / 2;
                    SetScalingFactor();
                }
            }
            get
            {
                return revenuesBase;
            }
        }

        public double HalfRevenuesBase
        {
            protected set
            {
                if (value != halfRevenuesBase)
                {
                    halfRevenuesBase = value;
                    OnPropertyChanged("HalfRevenuesBase");
                }
            }
            get
            {
                return halfRevenuesBase;
            }
        }

        void OnMaximumRevenueChanged(object sender, EventArgs e)
        {
            RevenuesBase = ChartingLibHelper.GetRoundedMaximum(Gizmos.MaximumRevenues, 2, 5);
        }

        void SetScalingFactor()
        {
            foreach (Gizmo gizmo in Gizmos)
            {
                gizmo.ScalingFactor = DisplayHeight / RevenuesBase;
            }
        }

        protected virtual void OnPropertyChanged(string propertyName)
        {
            if (PropertyChanged != null)
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
        }

        void OnTimerTick(object sender, EventArgs args)
        {
            int index = rand.Next(Gizmos.Count);
            Gizmos[index].Revenues *= 0.90 + 0.2 * rand.NextDouble();
        }
    }
}
